#include <stdio.h>
#include <stdlib.h>
#include <pam.h>
#include <gsl/gsl_matrix.h>
#include <math.h>
#include "libmat.h"

/* pgm2errormat. This program takes two pgm files as parameters, and writes
the absolute difference of the pictures's pixel values to stdout, following
the 'mat' matrix format.

    (float) out = abs((unsigned long) pixel1 - (unsigned long) pixel2);
*/

int main(int argn, char *argv[])
{
	FILE *input1, *input2;
	struct pam inpam1, inpam2;
	tuple **tuples1, **tuples2;
	int row, column;

	char *filename1, *filename2;
	int point1, point2; /* Points for each point error calculed */

	gsl_matrix *m_error;

	filename1 = argv[1];
	filename2 = argv[2];

	pnm_init(&argn, argv);


	/* Opening the FILEs */
	input1 = fopen(filename1,"r");
	if (input1 == NULL)
	{
		fprintf(stderr,"The file1 \"%s\" cannot be opened.\n",
			filename1);
		return 1;
	}
	input2 = fopen(filename2,"r");
	if (input1 == NULL)
	{
		fprintf(stderr,"The file2 \"%s\" cannot be opened.\n",
			filename2);
		return 1;
	}

	/* Reading the input files PAM header */
	tuples1 = pnm_readpam(input1, &inpam1, PAM_STRUCT_SIZE(tuple_type));
	tuples2 = pnm_readpam(input2, &inpam2, PAM_STRUCT_SIZE(tuple_type));
	if (inpam2.depth != 1)
	{
		fprintf(stderr,"The number of planes should be 1.\n");
		return 1;
	}

	if (inpam1.depth != 1 || inpam2.depth != 1)
	{
		fprintf(stderr,"The number of planes should be 1.\n");
		return 1;
	} else if (inpam1.width != inpam2.width ||
		inpam1.height != inpam2.height)
	{
		fprintf(stderr,"The two input images must be of same size.\n");
		return 1;
	}

	/* Memory allocated for the output matrix */
	m_error = gsl_matrix_alloc(inpam1.height, inpam1.width);

	for (row = 0; row < inpam1.height; row++) {
		for (column = 0; column < inpam1.width; ++column) {
			point1 = tuples1[row][column][0];
			point2 = tuples2[row][column][0];
			m_error->data[row * m_error->tda + column] =
				abs((int) point1 - (int)point2);
		}
	}

	/* Write the matrix file */
	mat_fwrite(stdout, m_error);

	/* Free all the memory */
	gsl_matrix_free(m_error);
	pnm_freepamarray(tuples1, &inpam1);
	pnm_freepamarray(tuples2, &inpam2);

	return 0;
}
